import 'package:flutter/material.dart';

/// Tipos de animaciones
enum AnimationType {
  normal,
  fadeIn,
}

/// Main class, [context] is the build context, [child] is the Page widget to be transition to
/// [animation] is the AnimationType enum from normal or fadeIn
/// [duration] is the elipsed time to perform the transion as Duration value.
/// [replacement] is a bool value which is used to set destination page as not returnable from the arrow on the appbar
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType? animation;
  final Duration? duration;
  final bool? replacement;

  /// Main class entry RouteTransitions
  RouteTransitions({
    required this.context,
    required this.child,
    this.animation = AnimationType.normal,
    this.duration = const Duration(milliseconds: 300),
    this.replacement = false,
  }) {
    //if (this.duration == null) {
    //  this.duration = new Duration(milliseconds: 300);
    //}
    switch (animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
      case null:
        break;
    }
    //
  }

  /// Push replacement de la pagina
  void _pushPage(Route route) => Navigator.push(context, route);

  /// Codigo de la transaccion com replacement
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  /// Codigo de la transaccion normal
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    (this.replacement!)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  /// Codigo de la transaccion con efecto fadeIn
  void _fadeInTransition() {
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: this.duration!,
        //transitionDuration: this.duration ?? Duration(milliseconds: 300),
        transitionsBuilder: (_, animation, __, child) {
          return FadeTransition(
              child: child,
              opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
                  CurvedAnimation(parent: animation, curve: Curves.easeOut)));
        });
    (this.replacement!)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }
}
